package flm.b4a.onlyme;

import java.io.DataOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.app.ActivityManager;
import android.app.admin.DevicePolicyManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import anywheresoftware.b4a.BA;

@BA.Author("F. Leneuf-Magaud/Informatix")
@BA.Version(1.0F)

@BA.Events(values = { "AdminEnabled", "AdminDisabled", "LockTaskModeEntering", "LockTaskModeExiting" })

@BA.ShortName("OnlyMe")
public class OnlyMe
{
	ComponentName deviceAdmin;
	DevicePolicyManager mDpm;

	public void Initialize(BA ba, String EvtPrefix)
	{
		AdminReceiver._ba = ba;
		String Prefix = EvtPrefix.toLowerCase(BA.cul);
		AdminReceiver._evtAdminEnabled = Prefix + "_adminenabled";
		AdminReceiver._evtAdminDisabled = Prefix + "_admindisabled";
		AdminReceiver._evtLockTaskModeEntering = Prefix + "_locktaskmodeentering"; 
		AdminReceiver._evtLockTaskModeExiting = Prefix + "_locktaskmodeexiting";

		deviceAdmin = new ComponentName(BA.applicationContext, AdminReceiver.class);
		mDpm = (DevicePolicyManager) BA.applicationContext.getSystemService(Context.DEVICE_POLICY_SERVICE);
	}

	/** Returns whether this application is registered with the system as a device administrator. */
	public boolean IsDeviceAdmin()
	{
		return mDpm.isAdminActive(deviceAdmin);
	}

	/** Opens the dialog asking the user to add this application to the system as a device administrator. */
	public void OpenAdminDialog(BA ba)
	{
		Intent intent = new Intent(DevicePolicyManager.ACTION_ADD_DEVICE_ADMIN);
		intent.putExtra(DevicePolicyManager.EXTRA_DEVICE_ADMIN, deviceAdmin);
		((BA) ba.sharedProcessBA.activityBA.get()).context.startActivity(intent);
	}

	/** Removes this application as a device administrator. */
	public void RemoveThisDeviceAdmin()
	{
		if (IsDeviceAdmin())
			mDpm.removeActiveAdmin(deviceAdmin);
	}

	/** Returns whether this application is registered with the system as a device owner. A device owner app is a special device admin that cannot be deactivated by the user once activated as a device admin and cannot be uninstalled. */
	public boolean IsDeviceOwner()
	{
		return mDpm.isDeviceOwnerApp(BA.packageName);
	}

	/** Sets this application as a device owner app on a rooted device. A device owner app is a special device admin that cannot be deactivated by the user once activated as a device admin and cannot be uninstalled.
	 *Returns whether the command succeeded.
	 *On an unrooted device, you have to run this command: adb shell dpm set-device-owner <I>&lt;your package name&gt;</I>/flm.b4a.onlyme.AdminReceiver */
	public boolean SetThisDeviceOwner() throws IOException, InterruptedException
	{
		String Cmd = "dpm set-device-owner " + BA.packageName + "/" + AdminReceiver.class.getName() + "\n";
		Process su = Runtime.getRuntime().exec("su");
		DataOutputStream outputStream = new DataOutputStream(su.getOutputStream());
		outputStream.writeBytes(Cmd);
		outputStream.flush();
		outputStream.writeBytes("exit\n");
		outputStream.flush();
		su.waitFor();
		return (su.exitValue() == 0);

	}

	/** Removes this application as a device owner. */
	public void RemoveThisDeviceOwner()
	{
		if (IsDeviceOwner())
			mDpm.clearDeviceOwnerApp(BA.packageName);
	}

	/** Returns whether this application is allowed to start the lock task mode. Returns False if the mode was not enabled with EnableLockTaskMode. */
	public boolean IsLockTaskPermitted()
	{
		return mDpm.isLockTaskPermitted(BA.packageName);
	}

	/** Allows this application to enter the lock task mode. In this mode, the app can be pinned without displaying a confirmation dialog and the user has no mean to exit.
	 *This function does nothing if the app is not a device owner. */
	public void EnableLockTaskMode()
	{
		if (mDpm.isDeviceOwnerApp(BA.packageName)) {
			mDpm.setLockTaskPackages(deviceAdmin, new String[] { BA.packageName });
		}
	}

	/** Pins the current activity and thus prevents the user from launching other apps, going to settings, or reaching the home screen.
	 *If the lock task mode is not enabled (IsLockTaskPermitted returns False), a dialog will ask the user to confirm the pinning and the user will be able to exit by holding the Home and Recents buttons. */
	public void Lock(BA ba)
	{
		if (!IsLocked()) {
			Activity _activity = ((BA) ba.sharedProcessBA.activityBA.get()).activity;
			_activity.startLockTask();
		}
	}

	public void Unlock(BA ba)
	{
		if (IsLocked()) {
			Activity _activity = ((BA) ba.sharedProcessBA.activityBA.get()).activity;
			_activity.stopLockTask();
		}
	}

	@SuppressWarnings("deprecation")
	public boolean IsLocked()
	{
		ActivityManager activityManager = (ActivityManager) BA.applicationContext.getSystemService(Context.ACTIVITY_SERVICE);
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) { // When SDK version >= 23
			int lockTaskMode = activityManager.getLockTaskModeState();
			return lockTaskMode != ActivityManager.LOCK_TASK_MODE_NONE; 
		}
		else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) { // When SDK version < 23
			return activityManager.isInLockTaskMode();
		}
		return false;
	}
}
